#include "crasm86.inc"

;EditLock86 1.0
;by Cassady Roop
;Edit-locks any un-locked BASIC program


dynamic_data	equ	$E300			;start of program data memory.  Needs at least 2K reserved.
prgm_name		equ	dynamic_data+0	;name of program to lock (type,length,8 byte name, not z-termed)
prgm_length		equ	dynamic_data+10	;length header of program.  2 bytes
prgm_firstbyte	equ	dynamic_data+12	;first byte of program data
prgm_secondbyte	equ	dynamic_data+13	;second byte of program data
listcounter		equ	dynamic_data+14	;a counter for reading the list
selected		equ	dynamic_data+15	;currently highlighted prgm name
vatlist_entries	equ	dynamic_data+16	;number of valid programs in the vatlist
vatlist			equ	dynamic_data+17	;stores returned list of program variables that are valid, unlocked prgm's


.org _asm_exec_ram

	nop
	jp ProgStart
	.dw 0
	.dw ShellTitle

ShellTitle:
	.db "EditLock 1.0",0
ProgStart:
	call _runindicoff
	call _flushallmenus
top:
	xor a
	ld (vatlist_entries), a				;clean slate
	ld hl, searchbase					;begin searching from this name
	call search							;searches the VAT for unlocked BASIC prgm's, and lists them in vatlist.
	ld a, (vatlist_entries)
	cp 0								;if search returns a 'none-found'
	jr nz, progs_present
	ld a, $FF
	ld (vatlist_entries), a				;FF will mean no prog found
progs_present:	
	xor a
	ld (selected), a					;upper left (entry 0) is selected
	call display						;displays user interface, and the vatlist, and current selection
	jp keyloop							;handles user input
	

search:								;search VAT for unlocked BASIC progs
	rst 20h							;move ten byte name at hl to OP1
	xor a							;a must be zero to search by type
	call _findAlphaUp				;returns the next highest program
	ret c							;if c, then there are no more vars to list
	ld hl, _OP1						;_findalphup returned the name in OP1
	ld de, prgm_name				;we will copy it here for safekeeping
	ld bc, $000A					;we want all ten bytes of OP1
	ldir							;block copy
	rst 10h							;_findsym
	ld a, b
	ex de, hl						;ld ahl, bde
	call fetch_header				;get length and first two bytes of program
	ld a, (prgm_firstbyte)			;the first byte of prgm data
	cp 0
	jr z, search_zero
	cp $8E
	jr nz, search_valid				;if not 00 or 8E, it is BASIC and unlocked
	ld a, (prgm_secondbyte)			;now we check the byte after the 8E
	cp $29
	jr z, search_invalid			;8E 29 is an edit-locked tokenized BASIC program
	cp $28
	jr z, search_invalid			;8E 28 is a compiled assembly program
	cp $27
	jr z, search_invalid			;8E 27 is a tokenized, uncompiled (AsmPrgm) assembly program
	jr search_valid					;anything else after 8E is acceptable
search_zero:
	ld a, (prgm_secondbyte)			;check byte after first zero
	cp 0
	jr z, search_invalid			;00 00 is an edit-locked plain-text BASIC program
search_valid:
	ld a, (vatlist_entries)			;current number of entries stored
	ld l, a
	ld h, 0							;hl = $00[vatlist_entries]
	call _mulHL10					;hl = hl*10
	ld de, vatlist					;start of the list
	add hl, de						;add the offset
	ex de, hl						;put the new entry address in de
	ld hl, prgm_name				;point to the name to store
	ld bc, $000A					;ten bytes
	ldir							;copy to the vatlist
	ld a, (vatlist_entries)			;get old number of entries
	inc a
	ld (vatlist_entries), a			;update it
	ld hl, prgm_name
	jr search						;reloop, using this name as a search base
search_invalid:						;if the program found was unacceptable
	ld hl, prgm_name				;the name of the unacceptable program
	jr search						;reloop, with next search based on this prgm name
	
	
display:							;display interface and prgm list
	call _clrLCD
	ld hl, $0000
	ld (_pencol), hl
	ld hl, titletext
	call _vputs						;display title
	ld hl, $004B
	ld (_pencol), hl
	ld hl, credittext
	call _vputs						;'by Cassady Roop'
	ld hl, $FC70					;row 7
	ld b, 16
display_lineloop:
	ld (hl), $FF					;8 dark pixels
	inc hl
	djnz display_lineloop			;do it 16 times for a whole line
	call makemenu					;make a menu at the bottom of the screen
	ld a, (vatlist_entries)
	cp $FF
	ret z							;nothing to display
	call highlight					;make black bar at selection
	ld hl, $0301
	ld (_pencol), hl
	xor a
	ld (listcounter), a				;clean start
disploop1:							;handles column 1
	ld a, (_penRow)					;current row coord
	add a, $06						;add one vWidth line
	ld b, a							;store it
	sub 50							;subtract row 56
	jr z, dispcol2					;if zero, we are at bottom of screen
	jp p, dispcol2					;if positive, we are at bottom of screen
	ld a, b							;recall new row value
	ld (_penRow), a					;write it back
	ld a, (vatlist_entries)
	ld b, a
	ld a, (listcounter)
	cp b
	jp z, disploops_done			;if they match, we are done with displaying name lists
	ld l, a							;loading listcounter into hl
	ld h, 0
	call _mulHL10					;multiply by ten
	ld de, vatlist
	add hl, de						;hl points to entry to display
	ld de, prgm_name
	ld bc, $000A
	ldir							;copy it to temp location
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b
	call z, inverse_on				;turn on inverse text if we are displaying the selected item
	ld hl, prgm_name+1				;skip first byte, which is a type byte.  Get length byte.
	ld b, (hl)						;get the length byte into b
	inc hl							;point to first char in name
	ld a, (_pencol)
	push af							;save column coord
disploop1_chars:
	ld a, (hl)						;get character to print
	push hl							;preserve pointer
	push bc							;preserve counter
	call _vputmap					;display char
	pop bc
	pop hl
	inc hl							;point to next char
	djnz disploop1_chars			;do until done with that name
	pop af
	ld (_pencol), a					;recall column coords
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b
	call z, inverse_off
	inc a
	ld (listcounter), a				;update the list progress counter
	jr disploop1					;reloop for the other names

dispcol2:							;display column 2
	ld hl, $0329
	ld (_pencol), hl
disploop2:							;handles column 2
	ld a, (_penRow)					;current row coord
	add a, $06						;add one vWidth line
	ld b, a							;store it
	sub 50							;subtract row 56
	jr z, dispcol3					;if zero, we are at bottom of screen
	jp p, dispcol3					;if positive, we are at bottom of screen
	ld a, b							;recall new row value
	ld (_penRow), a					;write it back
	ld a, (vatlist_entries)
	ld b, a
	ld a, (listcounter)
	cp b
	jp z, disploops_done			;if they match, we are done with displaying name lists
	ld l, a
	ld h, 0
	call _mulHL10					;multiply by ten
	ld de, vatlist
	add hl, de						;hl points to entry to display
	ld de, prgm_name
	ld bc, $000A
	ldir							;copy it to temp location
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b							;displaying selection?  Then invert
	call z, inverse_on
	ld hl, prgm_name+1				;skip first byte, which is a type byte.  Get length byte.
	ld b, (hl)						;get the length byte into b
	inc hl							;point to first char in name
	ld a, (_pencol)
	push af							;save column coord
disploop2_chars:
	ld a, (hl)
	push hl
	push bc
	call _vputmap					;display char
	pop bc
	pop hl
	inc hl							;point to next char
	djnz disploop2_chars			;do until done with that name
	pop af
	ld (_pencol), a					;recall column coords
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b
	call z, inverse_off
	inc a
	ld (listcounter), a				;update the list progress counter
	jr disploop2					;reloop for the other names

dispcol3:
	ld hl, $0351
	ld (_pencol), hl
disploop3:							;handles column 2
	ld a, (_penRow)					;current row coord
	add a, $06						;add one vWidth line
	ld b, a							;store it
	sub 50							;subtract row 56
	jr z, disploops_done			;if zero, we are at bottom of screen, and 3 columns is the max
	jp p, disploops_done			;if positive, we are at bottom of screen, and 3 columns is the max
	ld a, b							;recall new row value
	ld (_penRow), a					;write it back
	ld a, (vatlist_entries)
	ld b, a
	ld a, (listcounter)
	cp b
	jp z, disploops_done			;if they match, we are done with displaying name lists
	ld l, a
	ld h, 0
	call _mulHL10					;multiply by ten
	ld de, vatlist
	add hl, de						;hl points to entry to display
	ld de, prgm_name
	ld bc, $000A
	ldir							;copy it to temp location
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b							;are we displaying the selected item?
	call z, inverse_on				;turn on inverted text
	ld hl, prgm_name+1				;skip first byte, which is a type byte.  Get length byte.
	ld b, (hl)						;get the length byte into b
	inc hl							;point to first char in name
	ld a, (_pencol)
	push af							;save column coord
disploop3_chars:
	ld a, (hl)
	push hl
	push bc
	call _vputmap					;display char
	pop bc
	pop hl
	inc hl							;point to next char
	djnz disploop3_chars			;do until done with that name
	pop af
	ld (_pencol), a					;recall column coords
	ld a, (selected)
	ld b, a
	ld a, (listcounter)
	cp b
	call z, inverse_off				;turn off inverted text
	inc a
	ld (listcounter), a				;update the list progress counter
	jr disploop3					;reloop for the other names

inverse_on:							;turns on inverse text
	set textInverse, (IY+textFlags)
	ret
inverse_off:
	res textInverse, (IY+textFlags)
	ret


disploops_done:						;now we find and hightlight the selection
	ret

highlight:
	ld hl, $FC90					;row nine
	ld de, $0060					;six rows to add
	ld a, (selected)				;current selection
	push af
	sub 7
	jp p, highlight_col2			;if over 7, then move over a column first
	pop af
findloop:
	cp 0							;done?
	jr z, makeblock					;the correct point has been found.  Now draw a block there
	add hl, de						;add six lines
	dec a							;decrement counter
	jr findloop						;reloop
highlight_col2:
	inc hl
	inc hl
	inc hl
	inc hl							;offset hl to draw in column two
	inc hl
	pop af							;get selected item number
	push af
	sub 14
	jp p, highlight_col3			;if greater than 20, it is in column 3
	pop af
	sub 7							;start at top of column
	jr findloop
highlight_col3:
	inc hl
	inc hl
	inc hl
	inc hl							;offset hl to draw in the third column
	inc hl
	pop af
	sub 14							;start at top of column
	jr findloop
makeblock:	
	ld b, 6
makeblock_loop:						;makes a black block 6X32 pixels
	push hl
	ld (hl), $FF
	inc hl
	ld (hl), $FF
	inc hl
	ld (hl), $FF
	inc hl
	ld (hl), $FF
	inc hl
	ld (hl), %11110000
	pop hl
	ld de, $0010					;one row
	add hl, de
	djnz makeblock_loop				;loop until whole block drawn
	ret



makemenu:
	;*****make a menu at bottom of screen*****
	ld hl, $FF80			;start byte of row 56 in video mem
	ld b, 32					;initialize counter
HLineLoop:
	ld (hl), $FF				;darken 8 pixels
	inc hl					;next byte
	djnz HLineLoop			;recycle
	ld hl, $FFA0			;vidmem, first byte of row 58
	ld b, 6					;initialize line counter
	ld de, $0003
VLineLoop:
	ld (hl), %11000000				;move to vidmem
	add hl, de						;HL+3=HL
	ld (hl), %11000000
	add hl, de
	ld (hl), %01100000
	add hl, de
	ld (hl), %00011000
	add hl, de
	ld (hl), %00001100
	add hl, de
	ld (hl), %00000110
	inc hl					;next line
	djnz VLineLoop			;if not done, recycle
	ld hl, $FF8F
	ld a, %11111110				;remove unsightly piece of overshot
	ld (hl), a
	ld hl, $FF9F
	ld (hl), a
	ld hl, $3A05
	ld (_pencol), hl
	ld hl, locktext
	call _vputs					;'Lock'
;	ld hl, $3A1C
;	ld (_pencol), hl
;	ld hl, abouttext
;	call _vputs
	ld hl, $3A6A
	ld (_pencol), hl
	ld hl, quittext
	call _vputs					;'Quit'
	ret


keyloop:							;input loop	
	call _getkey					;get a keypress
	cp kexit
	jp z, _clrLCD					;exit key = quit
	cp kF1
	jp z, select					;F1 = commit
	cp kF5
	jp z, _clrLCD					;F5 = quit
	cp kenter
	jp z, select					;ENTER = commit
	cp kup
	jp z, up						;UP = move selection up
	cp kdown
	jp z, down						;DOWN = move selection down
	cp kleft
	jp z, left
	cp kright
	jp z, right
	jr keyloop						;reloop

up:									;move selection up
	ld a, (vatlist_entries)
	cp $FF
	jr z, keyloop					;if there are no progs, stop
	ld a, (selected)				;get current highlighted item
	cp 0
	jr z, keyloop					;if already at the top, don't do anything
	dec a
	ld (selected), a				;new selection value
	call display					;redraw the display
	jr keyloop						;back to user input

down:
	ld a, (vatlist_entries)
	cp $FF
	jr z, keyloop					;if there are no progs, stop
	ld b, a
	ld a, (selected)
	inc a							;the count starts at zero instead of one, so inc it
	cp b
	jr z, keyloop					;don't exceed the number of vatlist entries
	cp 21		;*****				;max value for the page
	jr z, keyloop					;can't go off the page
	ld (selected), a				;new selection value (already incremented)
	call display					;redraw
	jr keyloop						;return to user input loop

left:
	ld a, (vatlist_entries)
	cp $FF
	jr z, keyloop					;stop if there are no valid progs
	ld a, (selected)				;get current selection
	sub 7							;subtracting seven moves left one column
	jp m, keyloop					;if the number is negative, it goes off the screen
	ld (selected), a				;load new value to selection counter
	call display					;redraw
	jr keyloop						;return to user input loop

right:
	ld a, (vatlist_entries)
	cp $FF
	jr z, keyloop					;stop if there are no valid progs
	ld a, (selected)				;get current selection
	add a, 7						;add 7 to move right one column
	ld b, a							;hold that number...
	sub 21							;21 is highest allowable selection number on the page
	jp p, keyloop					;if it is positive, a move to the right goes off the page
	ld a, (vatlist_entries)			;number of programs
	ld c, a
	ld a, b							;the selection number + 7 that we stored earlier
	sub c							;subtract the number of programs from the new selection value
	jp p, keyloop					;if positive, the selection is higher than the number of valid programs
	ld a, b							;recall it again
	ld (selected), a				;save it
	call display					;redraw
	jp keyloop						;return to user input loop

select:
	ld a, (selected)				;get selection
	ld b, a
	ld a, (vatlist_entries)
	cp $FF
	jp z, keyloop					;stop if there are no programs
	sub b							;entries-selection
	jr z, select_continue			;if zero, they match
	jp m, keyloop					;if negative, selection does not exist.  If positive, it does.
select_continue:
	ld a, (selected)
	ld l, a
	ld h, 0
	call _mulHL10					;HL*10
	ld de, vatlist
	add hl, de						;hl now points to entry of selected program
	ld de, prgm_name
	ld bc, $000A
	ldir							;copy the entry to prgm_name
	call _clrLCD
	ld hl, $0630
	ld (_pencol), hl
	ld hl, confirmtext
	call _vputs						;'CONFIRM'
	ld hl, $1800
	ld (_pencol), hl
	ld hl, warningtext
	call _vputs						;'This action is permanent'
	ld hl, $1E00
	ld (_pencol), hl
	ld hl, asktext
	call _vputs						;'Edit-lock program "'
	ld hl, prgm_name+1
	ld b, (hl)						;name length
	inc hl
select_nameloop:					;display the name of the selected program
	ld a, (hl)						;get a character
	push bc
	push hl
	call _vputmap					;display character
	pop hl
	pop bc
	inc hl
	djnz select_nameloop			;loop until name is displayed
	ld a, 34						;a quote char
	call _vputmap					;close the quote around the displayed program name
	ld a, '?'
	call _vputmap					;put a question mark at the end
	ld hl, $2C2A
	ld (_pencol), hl
	ld hl, yestext
	call _vputs						;'F1- Continue'
	ld hl, $322A
	ld (_pencol), hl
	ld hl, canceltext
	call _vputs						;'F5- Cancel'
select_keyloop:
	call _getkey
	cp kF1
	jr z, select_confirmed			;lock the program
	cp kF5
	jr nz, select_keyloop			;reloop if not F1 or F5
	call display					;redraw
	jp keyloop						;back to user input if user canceled lock procedure
select_confirmed:
	ld a, (selected)				;get highlighted item number
	ld l, a
	ld h, 0							;put into hl
	call _mulHL10					;hl = hl * 10
	ld de, vatlist
	add hl, de						;hl now points to name of selected program in the vatlist
	ld de, prgm_name				;destination
	ld bc, $000A					;ten bytes
	ldir							;copy
	jr lock							;lock it




lock:									;edit-lock the program in OP1.
	ld hl, prgm_name
	rst 20h								;move it to OP1
	rst 10h								;_findsym
	ld a, b
	ex de, hl							;ld ahl, bde
	call fetch_header					;get the length header and first two bytes of program data
	ld hl, temp_name-1
	rst 20h								;put temp name in OP1
	ld hl, (prgm_length)				;get original prgm length
	ld a, (prgm_firstbyte)				;first data byte of program
	cp 0
	jr z, lock_1addone					;if first byte=0, editlocking will add one byte
	inc hl								;if not, we will add two...
lock_1addone:
	inc hl								;we now have the proper length for the temp program
	call _createprog					;allocate hl bytes for temp program with name in OP1
	ld a, b
	ex de, hl							;ld ahl, bde
	call $4C3F							;increment AHL twice to skip length bytes
	push af								;save a from ahl
	ld a, (prgm_firstbyte)
	cp 0
	jr z, lock_2addone					;if 0, then increment only once
	pop af
	call _inc_ahl
	push af
lock_2addone:
	pop af
	call _inc_ahl						;we have now incremented into the dest. program far enough
	call _set_abs_dest_addr				;we will copy to the temp program
	ld hl, prgm_name
	rst 20h								;move name to OP1
	rst 10h								;_findsym
	ld a, b
	ex de, hl							;ld ahl, bde
	call $4C3F							;skip length bytes
	call _set_abs_src_addr				;we will copy from here
	xor a								;zero out the a in ahl
	ld hl, (prgm_length)				;original length
	call _set_mm_num_bytes				;copy length
	call _mm_ldir						;block copy
	ld hl, temp_name-1
	rst 20h								;move name to OP1
	rst 10h								;_finsym
	ld a, b
	ex de, hl							;ld ahl, bde
	add a, 64							;set bit 6, it is a RAM page
	out (5), a							;swap in a RAM page
	inc a
	out (6), a							;swap in next RAM page
	ld de, $4000
	add hl, de							;offset into first RAM page window, now points to program
	ld a, (hl)
	ld (prgm_length), a					;new program's length
	inc hl
	ld a, (hl)
	ld (prgm_length+1), a				;we now have both bytes saved
	inc hl								;points to first data byte of new prog, which we will modify
	ld a, (prgm_firstbyte)				;first byte of original prog
	cp 0
	jr nz, tokenized					;if starts with 0, then it is plaintext, otherwise tokenized
plaintext:								;set it to 'plaintext, edit-locked' ($00 $00)
	ld (hl), a							;set first byte to zero.  Second byte is already a zero.  Done edit-locking.
	jr editlocked
tokenized:								;set it to 'tokenized, edit-locked' ($8e $29)
	ld (hl), $8E
	inc hl
	ld (hl), $29
editlocked:								;editlocking is done.  Now copy back to original prgm name.
	ld a, $0d
	out (5), a							;put ROM page 0D back so we can make ROM calls
	ld hl, prgm_name					;the original program
	rst 20h								;move name to OP1
	rst 10h								;_findsym
	call _delvar						;delete the old program var
	ld hl, prgm_name
	rst 20h								;put the name back in OP1
	ld hl, (prgm_length)				;get new program length
	call _createprog					;make a new program under original name.
	ld a, b
	ex de, hl							;ld ahl, bde
	call $4C3F							;AHL+2, get past length bytes
	call _set_abs_dest_addr				;we're gonna put the old data and edit-lock codes in to the new program
	ld hl, temp_name-1
	rst 20h								;move to OP1
	rst 10h								;_findsym
	ld a, b
	ex de, hl							;ld ahl, bde
	call $4C3F							;skip length bytes
	call _set_abs_src_addr				;get the old data location (in the temp var)
	xor a
	ld hl, (prgm_length)				;get the length to copy
	call _set_mm_num_bytes				;set it
	call _mm_ldir						;copy
	ld hl, temp_name-1
	rst 20h								;move name to OP1
	rst 10h								;_findsym
	call _delvar						;delete the temporary program var
success:
	call _clrLCD
	ld hl, $182A
	ld (_pencol), hl
	ld hl, successfultext
	call _vputs
	call _getkey						;wait for keypress
	call _clrlcd
	jp top								;restart
	

fetch_header:							;finds program var at ahl, stores length header and first 2 bytes of data
	add a, 64							;set bit 6, so it represents a RAM page and not ROM
	out (5), a							;load RAM page to swapping area $4000-$7FFF
	inc a								;next page
	out (6), a							;load next RAM page to swapping area $8000-$BFFF
	ld de, $4000						;offset to first swapping area
	add hl, de							;add it to hl, to make a valid pointer in hl
	ld a, (hl)
	ld (prgm_length), a					;first byte of length header
	inc hl
	ld a, (hl)
	ld (prgm_length+1), a				;second byte of length header
	inc hl
	ld a, (hl)
	ld (prgm_firstbyte), a				;first byte of data
	inc hl
	ld a, (hl)
	ld (prgm_secondbyte), a				;second byte of data
	ld a, $0D							;ROM page containing call table
	out (5), a							;put it back so that I can use ROM calls again
	ret									;done






asktext:
	.db "Edit-lock program ",34,0		;34 is a quote char
titletext:
	.db "EditLock 1.0",0
credittext:
	.db "by Cassady Roop",0
yestext:
	.db "F1- Continue",0
canceltext:
	.db "F5- Cancel",0
locktext:
	.db "Lock",0
quittext:
	.db "Quit",0
successfultext:
	.db "SUCCESSFUL",0
confirmtext:
	.db "CONFIRM",0
warningtext:
	.db "This action is permanent.",0
temp_name:
	.db 7,"BElockT"
test_progname:
	.db 4,"TEST"
searchbase:								;base name to search from in alphabetic search of VAT for program vars
	.db $12,$08,"AAAAAAAA"				;type $12 (prgm), length $08, a bunch of a's.



	



.end
